import logging
from . import _az_facade as az

log = logging.getLogger(__name__)


def prepare(data: dict):
    subscription_id = data["subscriptionId"]
    az.set_subscription(subscription_id)

    applicationId = data.get("applicationId")
    if applicationId:
        log.info("Initializing with configured SP.")
        applicationKey = data.get("applicationKey")
        # print("Login az CLI...")
        ret = az.login(applicationId, applicationKey, data.get("directoryId"))
        # print(ret)
    # _az_facade.login(provider['applicationId'], provider['applicationKey'], directory_id)
    # _az_facade.set_subscription(subscription_id)
    else:
        log.info("No configured SP found. Using Azure CLI stored login.")

    ret = az.test_credential()
    # print(ret)



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import subprocess
import json
import logging

log = logging.getLogger(__name__)

_subscription = None


def _az(command: str, silent: bool = False) -> dict:
    log.debug(command)
    args = command.split(" ")
    p = subprocess.run(args, capture_output=True, text=True, timeout=None, check=False)
    if p.returncode:
        if silent:
            pass
        else:
            log.error("RETURN: " + str(p.returncode))
            log.error("STDOUT: " + p.stdout)
            log.error("STDERR: " + p.stderr)
            raise subprocess.CalledProcessError(p.returncode, p.args, output=p.stdout, stderr=p.stderr)
    else:
        if p.stdout:
            try:
                return json.loads(p.stdout)
            except Exception as e:
                log.error("Fail parsing response %s", e)
                log.warning("Payload: %s", p.stdout)


def login(username: str, password: str, tenant_id: str):
    return _az(f"az login --service-principal -u {username} -p {password} --tenant {tenant_id}")


def _formalize_tags(tags: dict) -> str:
    list_tags = []
    if tags:
        for k, v in tags.items():
            list_tags.append(f"{k}={v}")
    return " ".join(list_tags)


def set_subscription(id: str):
    global _subscription
    _subscription = id
    return _az(f"az account set --subscription {id}")


def test_credential():
    return _az(f"az account get-access-token")


def locations():
    return _az("az account list-locations")


class resource_group:
    @staticmethod
    def create(name: str, location: str, tags: dict = None):
        tags_str = _formalize_tags(tags)
        return _az(f"az group create --location {location} --name {name} --tags {tags_str}")

    @staticmethod
    def delete(name: str):
        exists = _az(f"az group exists --name {name}")
        if exists:
            _az(f"az group delete --name {name} --yes")
            return True
        return False


class _aad_group_member:
    @staticmethod
    def list(id_or_display_name: str):
        return _az(f"az ad group member list --group {id_or_display_name}")

    @staticmethod
    def add(id_or_display_name: str, member_id: str):
        return _az(f"az ad group member add --group {id_or_display_name} --member-id {member_id}")


class _aad_group:
    member = _aad_group_member

    @staticmethod
    def create(display_name: str, mail_nickname: str, description: str = None):
        cmd = f"az ad group create --display-name {display_name} --mail-nickname {mail_nickname}"
        if description:
            cmd += f" --description {description}"
        return _az(cmd)

    @staticmethod
    def get(id_or_display_name: str):
        return _az(f"az ad group show --group {id_or_display_name}", silent=True)

    @staticmethod
    def delete(id_or_display_name: str):
        return _az(f"az ad group delete --group {id_or_display_name}")

    @staticmethod
    def list():
        return _az(f"az ad group list")


class _aad_user:
    @staticmethod
    def create(display_name: str, password: str, principal_name: str):
        return _az(
            f"az ad user create --display-name {display_name} --password {password} --user-principal-name {principal_name}"
        )

    @staticmethod
    def get(id_or_principal_name: str):
        return _az(f"az ad user show --id {id_or_principal_name}", silent=True)

    @staticmethod
    def delete(id: str):
        return _az(f"az ad user delete --id {id}")


class aad:
    group = _aad_group
    user = _aad_user


class _vnet:
    @staticmethod
    def get(id: str):
        return _az(f"az network vnet show --ids {id}")

    @staticmethod
    def create(rg_name: str, name: str, address_prefix: str):
        return _az(f"az network vnet create -g {rg_name} -n {name} --address-prefix {address_prefix}")

    @staticmethod
    def delete(rg_name: str, name: str, address_prefix: str):
        return _az(f"az network vnet create -g {rg_name} -n {name} --address-prefix {address_prefix}")

    @staticmethod
    def list():
        return _az("az network vnet list")


class _nsg:
    @staticmethod
    def create(rg_name: str, name: str, location: str, tags: list[str] = None):
        tags_str = _formalize_tags(tags)
        return _az(f"az network nsg create -g {rg_name} -n {name} --location {location} --tags {tags_str}")

    @staticmethod
    def delete_by_id(id: str):
        return _az(f"az network nsg delete --ids {id}")

    @staticmethod
    def delete(rg_name: str, name: str):
        return _az(f"az network nsg delete -g {rg_name} -n {name}")


class _subnet:
    @staticmethod
    def create(rg_name: str, vnet_name: str, name: str, cidr: str, nsg_name: str):
        return _az(
            f"az network vnet subnet create -g {rg_name} --vnet-name {vnet_name} -n {name} --address-prefixes {cidr} --network-security-group {nsg_name}"
        )

    @staticmethod
    def delete_by_id(id: str):
        return _az(f"az network vnet subnet delete --ids {id}")

    @staticmethod
    def delete(rg_name: str, vnet_name: str, name: str):
        return _az(f"az network vnet subnet delete -g {rg_name} --vnet-name {vnet_name} -n {name}")

    @staticmethod
    def list(rg_name: str, vnet_name: str):
        return _az(f"az network vnet subnet list -g {rg_name} --vnet-name {vnet_name}")


class _public_ip:
    @staticmethod
    def create(rg_name: str, name: str, location: str):
        return _az(
            f"az network public-ip create -g {rg_name} -n {name} --location {location} --allocation-method Static"
        )

    @staticmethod
    def get(rg_name: str, name: str):
        return _az(f"az network public-ip show -g {rg_name} -n {name}")

    @staticmethod
    def delete(rg_name: str, name: str):
        return _az(f"az network public-ip delete -g {rg_name} -n {name}")


class network:
    vnet = _vnet
    nsg = _nsg
    subnet = _subnet
    public_ip = _public_ip


def adjust_tags(parameters) -> str:
    return parameters


from hcs_core.plan import context
from azure.identity import AzureCliCredential
from azure.mgmt.network import NetworkManagementClient
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.msi import ManagedServiceIdentityClient
from azure.mgmt.storage import StorageManagementClient


def network_management_client() -> NetworkManagementClient:
    credentials = AzureCliCredential()
    return NetworkManagementClient(credentials, _subscription)


def resource_management_client() -> ResourceManagementClient:
    credentials = AzureCliCredential()
    return ResourceManagementClient(credentials, _subscription)


def managed_identity_client() -> ManagedServiceIdentityClient:
    credentials = AzureCliCredential()
    return ManagedServiceIdentityClient(credentials, _subscription)


def storage_management_client() -> StorageManagementClient:
    credentials = AzureCliCredential()
    return StorageManagementClient(credentials, _subscription)



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from hcs_core.plan import actions
from azure.core.exceptions import ResourceNotFoundError
import random
from . import _az_facade as az


def deploy(data: dict, state: dict) -> dict:
    group = data["group"]
    email = data["email"]
    domain_name = data["domainName"]

    pname = _convert_email_to_principle(email, domain_name)
    display_name = pname
    password = _generate_password()
    principal_name = pname
    ret = az.aad.user.create(display_name, password, principal_name)
    az.aad.group.member.add(group, ret["id"])
    ret["password"] = password
    return ret


def _convert_email_to_principle(email: str, domain_name: str) -> str:
    name = email.replace("@", "_")
    return name + "@" + domain_name


def _generate_password() -> str:
    ret = random.choices("ABCDEFGHJKMNPQRSTUVWXY")
    ret += random.choices("abcdefghjkmnpqrstuvwxy23456789", k=10)
    ret += random.choices("!@$%&*")
    return "".join(ret)


def refresh(data: dict, state: dict) -> dict:
    if state:
        id = state["id"]
        ret = az.aad.user.get(id)
        if ret:
            ret["password"] = state.get("password")
            return ret
    email = data["email"]
    domain_name = data["domainName"]
    pname = _convert_email_to_principle(email, domain_name)
    return az.aad.user.get(pname)


def decide(data: dict, state: dict):
    if not state.get("password"):
        return actions.recreate
    return actions.skip


def destroy(data: dict, state: dict, force: bool) -> dict:
    id = state["id"]
    ret = az.aad.user.delete(id)
    return ret


def eta(action: str, data: dict, state: dict):
    return "1m"



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from hcs_core.plan import actions, PluginException
from hcs_ext_daas.av_helper import fs_copy, fs_delete, fs_refresh

import logging

log = logging.getLogger(__name__)


def deploy(data: dict, state: dict, save_state) -> dict:
    try:
        deployment = fs_copy(data, state)
    except Exception as e:
        raise PluginException("Exception while copying AV files to storage account.") from e
    return deployment


def refresh(data: dict, state: dict) -> dict:
    return fs_refresh(data, state)


def decide(data: dict, state: dict):
    _is_skip = True
    if state:
        for _rec in state:
            if _rec.get("status") and _rec.get("status") != "success":
                _is_skip = False
                break
    if _is_skip:
        return actions.skip


def destroy(data: dict, state: dict, force: bool) -> dict:
    return fs_delete(data)
    # TODO: Implement wait


def eta(action: str, data: dict, state: dict):
    return "3m"



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from hcs_core.plan import actions
from azure.core.exceptions import ResourceNotFoundError
from ._az_facade import resource_management_client as client, adjust_tags


def deploy(data: dict, state: dict) -> dict:
    name = data["name"]
    parameters = adjust_tags(data["parameters"])
    return client().resource_groups.create_or_update(name, parameters)


def refresh(data: dict, state: dict) -> dict:
    name = data["name"]
    try:
        return client().resource_groups.get(name)
    except ResourceNotFoundError:
        pass


def decide(data: dict, state: dict):
    return actions.skip


def destroy(data: dict, state: dict, force: bool) -> dict:
    name = data["name"]
    try:
        return client().resource_groups.begin_delete(name).wait()
    except ResourceNotFoundError:
        pass


def eta(action: str, data: dict, state: dict):
    if action == actions.delete:
        return "5m"
    return "1m"
